SystemOrganization addCategory: #'System-CommandLine'!

!Color methodsFor: '*System-CommandLine' stamp: 'CamilloBruni 7/1/2012 15:03'!
closestXTermPixelValue
	| index |
	"Return the nearest approximation to this color for 256 xterm color."

	self saturation < 0.2 ifTrue: [
		"colors 232-255 are a grayscale ramp, intentionally leaving out black and white"
		index := (self green * 25) asInteger.
		index = 0 ifTrue: [ ^ 0 ].
		index = 25 ifTrue: [ ^ 16 ].
		^ index + 232 ].
	
	"compute nearest entry in the 6*6*6 color cube"
	^ 16 + (self red * 6*6*5) asInteger + (self green * 6*5) asInteger + (self blue * 5) asInteger
! !

!CodeImporter methodsFor: '*System-CommandLine' stamp: 'CamilloBruni 6/14/2012 23:26'!
evaluate
	" stripped down version of evaluateDeclarations"
	| value |
	
	self codeDeclarations do: [ :decl | value := decl import ].
	self flushChangesFile.
	
	^value! !

Object subclass: #CommandLineArguments
	instanceVariableNames: 'arguments'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-CommandLine'!

CommandLineArguments class
	instanceVariableNames: 'singleton'!

!CommandLineArguments commentStamp: '<historical>' prior: 0!
The CommandLineArguments represents the arguments passed to the image.
In the following case,

	$PHARO_VM myImage.image --foo bar
	
`CommandLineArguments default` contains {'--foo'. 'bar'}.!

CommandLineArguments class
	instanceVariableNames: 'singleton'!

!CommandLineArguments class methodsFor: 'instance creation' stamp: 'CamilloBruni 5/2/2012 11:41'!
default
	^ singleton ifNil: [singleton := self new initialize]! !

!CommandLineArguments class methodsFor: 'instance creation' stamp: 'CamilloBruni 5/1/2012 20:00'!
withArguments: aCollection
	^ self basicNew 
		initializeWithArguments: aCollection;
		yourself! !

!CommandLineArguments methodsFor: 'accessing' stamp: 'CamilloBruni 5/2/2012 12:33'!
allFilesWithExtension: anExtension
	^ self arguments select: [ :arg|
		arg endsWith: anExtension ]! !

!CommandLineArguments methodsFor: 'accessing' stamp: 'CamilloBruni 5/1/2012 19:57'!
argumentAt: index
	^ arguments at: index ! !

!CommandLineArguments methodsFor: 'accessing' stamp: 'CamilloBruni 4/28/2012 19:42'!
arguments
	^ arguments! !

!CommandLineArguments methodsFor: 'testing' stamp: 'CamilloBruni 5/13/2012 19:47'!
commandLineArguments
	"self commandLineArguments"
	
	| documentPath args |
	
	args := OrderedCollection withAll: Smalltalk arguments.
	documentPath := Smalltalk vm documentPath.
	documentPath isEmptyOrNil
		ifFalse: [ args addFirst: documentPath ].
 	^ args.! !

!CommandLineArguments methodsFor: 'copying' stamp: 'CamilloBruni 5/1/2012 20:07'!
copySubcommand
	"return a new copy of this CommandLine without the first arguments"
	^ self class withArguments: arguments allButFirst! !

!CommandLineArguments methodsFor: 'testing' stamp: 'CamilloBruni 5/1/2012 20:50'!
hasArguments
	^ arguments size > 0! !

!CommandLineArguments methodsFor: 'testing' stamp: 'CamilloBruni 5/2/2012 12:55'!
hasFileWithExtension: aFileExtension
	"return true if the first argument has the given file extension"
	^ self arguments anySatisfy: [ :arg| 
		arg endsWith: aFileExtension].! !

!CommandLineArguments methodsFor: 'testing' stamp: 'CamilloBruni 10/13/2012 15:13'!
hasOption: aString
	| option |
	option := (aString size = 1
					ifTrue: [ '-']
					ifFalse: [ '--']), aString.
	(self arguments includes: option) ifTrue: [ ^ true ].
	option := option,'='.
	^ self arguments anySatisfy: [ :arg| arg beginsWith: option ].! !

!CommandLineArguments methodsFor: 'testing' stamp: 'CamilloBruni 5/2/2012 14:07'!
includesSubCommand: aName
	self withFirstArgument: [ :arg|
		arg = aName ifTrue: [ ^ true ]].
	^ false! !

!CommandLineArguments methodsFor: 'testing' stamp: 'CamilloBruni 5/13/2012 19:47'!
initialize
	
	| documentPath  |
	
	arguments := OrderedCollection withAll: Smalltalk arguments.
	documentPath := Smalltalk vm documentPath.
	documentPath isEmptyOrNil
		ifFalse: [ arguments addFirst: documentPath ].! !

!CommandLineArguments methodsFor: 'initialize-release' stamp: 'CamilloBruni 5/1/2012 20:01'!
initializeWithArguments: aCollection
	super initialize.
	arguments := aCollection.! !

!CommandLineArguments methodsFor: 'testing' stamp: 'CamilloBruni 10/13/2012 15:19'!
longOptionAt: aString
	| option optionStart |
	optionStart := '--', aString, '='.
	option := self arguments 
		detect: [ :arg| arg beginsWith: optionStart ] 
		ifNone: [ ^ nil ].
	^ (option splitOn: '=') second! !

!CommandLineArguments methodsFor: 'accessing' stamp: 'CamilloBruni 10/13/2012 15:24'!
optionAt: aString
	^ (aString size = 1)
		ifTrue: [ self shortOptionAt: aString ]
		ifFalse: [ self longOptionAt: aString ]! !

!CommandLineArguments methodsFor: 'testing' stamp: 'CamilloBruni 10/13/2012 15:24'!
shortOptionAt: aString
	^ arguments at: (arguments indexOf: '-', aString) + 1! !

!CommandLineArguments methodsFor: 'testing' stamp: 'CamilloBruni 4/28/2012 19:54'!
withFirstArgument: aBlock
	self arguments ifEmpty: [ ^ self ].
	^ aBlock value: self arguments first! !

Object subclass: #CommandLineHandler
	instanceVariableNames: 'commandLine stdout stderr'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-CommandLine'!

!CommandLineHandler commentStamp: '<historical>' prior: 0!
A CommandLineHandler is activated by the CommandLine. 

The responsible handler with the highest priority is selected and its instance-side method #activate is invoked.

By default the handlers are selected by their class name. In the following shell invocation the FooHandler is chosen:

	$PATH_TO_VM myImage.image FooHandler

A handler may provide a short name with the class-side #commandName method. If the FooHandler defined #commandName returning 'foo' it would be activated with the following shell invocation:
	
	$PATH_TO_VM myImage.image foo
	
For more sophisticated handler selection the CommandLineHandler should implement the #isResponsibleFor: class-side method. An instance of the current command line options is passed to this method which should then return a boolean.

Between all the responsible handlers the one with the highes #priority is chosen. To change the priority overwrite the class-side accessor.
!

CommandLineHandler subclass: #BasicCodeLoader
	instanceVariableNames: 'sourceFiles'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-CommandLine'!

!BasicCodeLoader commentStamp: '<historical>' prior: 0!
CodeLoader provides a way to load smalltalk source file from local.

Example: 

CodeLoader new 
	loadSourceFiles: (Array with: 
		(#file:, 
		FileDirectory default fullName, 
		'/useless.st')); 
	installSourceFiles !

!BasicCodeLoader class methodsFor: 'accessing' stamp: 'CamilloBruni 5/2/2012 11:35'!
commandName
	^ 'st'! !

!BasicCodeLoader class methodsFor: 'accessing' stamp: 'CamilloBruni 5/3/2012 14:14'!
description
	^ 'handles .st source files'! !

!BasicCodeLoader class methodsFor: 'handler selection' stamp: 'CamilloBruni 5/2/2012 11:31'!
isResponsibleFor: aCommandLine
	"This handler is reponsible only for .st files"
	(aCommandLine hasFileWithExtension: '.st')
		ifTrue: [ ^ true ].
	^ super isResponsibleFor: aCommandLine! !

!BasicCodeLoader methodsFor: 'activation' stamp: 'CamilloBruni 10/13/2012 15:50'!
activate
	self activateHelp.
	self loadSourceFiles: (self commandLine allFilesWithExtension: 'st').
	self installSourceFiles.
	(self commandLine hasOption: 'quit')
		ifTrue: [ self exitSuccess ].! !

!BasicCodeLoader methodsFor: 'private' stamp: 'CamilloBruni 4/28/2012 21:48'!
handleError: error	
	
	"for syntax errors we can used the default action"	"otherwise resignal it"
	(error isKindOf: SyntaxErrorNotification)
		ifTrue: [ error defaultAction ]
		ifFalse: [ error pass ]! !

!BasicCodeLoader methodsFor: 'private' stamp: 'EstebanLorenzano 4/4/2012 10:38'!
handleError: error reference: aReference
	"Print a header before failing on an errro / syntax notification from the the script loaded by the given request"
	|stderr|
	
	"spit out a warning if in headless mode, otherwise a debugger will popup"
	Smalltalk isHeadless 
		ifTrue: [ self inform: 'Errors in script loaded from ', aReference fullName ].
																														
	(error isKindOf: SyntaxErrorNotification)
		"for syntax errors we can used the default action"
		ifTrue: [ error defaultAction ]
		"otherwise resignal it"
		ifFalse: [ error pass ]! !

!BasicCodeLoader methodsFor: 'installing' stamp: 'CamilloBruni 6/14/2012 23:15'!
handleErrorsDuring: aBlock reference: aReference 
	
	aBlock
		on: Error, ParserNotification
		do: [ :e| 	self handleError: e reference: aReference ].
	! !

!BasicCodeLoader methodsFor: 'installing' stamp: 'CamilloBruni 6/14/2012 23:29'!
installSourceFile: aReference
	"Install the the source file given by aFileReference"
	| codeImporter |

	" parse the code given in the source file"
	aReference readStreamDo: [ :stream |
		self 
			handleErrorsDuring: [ 
				codeImporter := CodeImporter fileStream: stream.
				codeImporter parseDeclarations ]
			reference: aReference].
	
	"evaluate the declarations in a second step to allow for properly closing the source file stream"
	self 
		handleErrorsDuring: [ codeImporter evaluate ]
		reference: aReference.! !

!BasicCodeLoader methodsFor: 'installing' stamp: 'EstebanLorenzano 4/4/2012 10:31'!
installSourceFiles
	"Install the previously loaded source files"

	sourceFiles ifNil: [ ^self ].
	[ sourceFiles do: [ :reference | self installSourceFile: reference ] ]
	ensure: [ sourceFiles := nil ].
	! !

!BasicCodeLoader methodsFor: 'loading' stamp: 'CamilloBruni 5/1/2012 20:51'!
loadSourceFiles: anArray
	"Load all the source files in the given array."

	sourceFiles := anArray collect: [ :each | 
						(FileSystem disk resolve: each) asFileReference ].
! !

BasicCodeLoader subclass: #EvaluateCommandLineHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-CommandLine'!

!EvaluateCommandLineHandler commentStamp: '<historical>' prior: 0!
Usage: eval [--help] <smalltalk expression>
	--help    list this help message
	<smallltalk expression>  a valid Smalltalk expression which is evaluated and 
	                         the result is printed on stdout

Documentation:
A CommandLineHandler that reads a string from the command line, outputs the evaluated result and quits the image. 

This handler either evaluates the arguments passed to the image:
	$PHARO_VM my.image eval  1 + 2
	
or it can read directly from stdin:

	echo "1+2" | $PHARO_VM my.image eval !

!EvaluateCommandLineHandler class methodsFor: 'accessing' stamp: 'CamilloBruni 4/28/2012 22:48'!
commandName
	^ 'eval'! !

!EvaluateCommandLineHandler class methodsFor: 'accessing' stamp: 'CamilloBruni 5/3/2012 14:14'!
description
	^ 'directly evaluates passed in one line scripts'! !

!EvaluateCommandLineHandler class methodsFor: 'handler selection' stamp: 'CamilloBruni 7/4/2012 00:20'!
isResponsibleFor: commandLineArguments
	"directly handle top-level -e and --evaluate options"
	commandLineArguments withFirstArgument: [ :arg| 
		(#('-e' '--evaluate') includes: arg)
			ifTrue: [ ^ true ]].
	
	^ commandLineArguments includesSubCommand: self commandName! !

!EvaluateCommandLineHandler methodsFor: 'activation' stamp: 'CamilloBruni 10/13/2012 15:49'!
activate
	self activateHelp.
	self arguments ifEmpty: [ ^ self evaluateStdIn ].
	self evaluateArguments.
	self quit.! !

!EvaluateCommandLineHandler methodsFor: 'commands' stamp: 'CamilloBruni 6/23/2012 22:33'!
evaluate: aStream
		
	[ self stdout print: (Compiler evaluate: aStream); cr ]
		on: Error, ParserNotification
		do: [ :e| 	self handleError: e  ].! !

!EvaluateCommandLineHandler methodsFor: 'activation' stamp: 'CamilloBruni 6/23/2012 22:46'!
evaluateArguments
	| args |
	args :=  self arguments joinUsing: Character space.
	args ifEmpty: [ ^ self ].
	self evaluate: args! !

!EvaluateCommandLineHandler methodsFor: 'activation' stamp: 'CamilloBruni 6/23/2012 22:45'!
evaluateStdIn
	| stdin |
	stdin := FileStream stdin ifNil: [ ^ self ].
	stdin atEnd  ifTrue: [ ^ self ].
	
	self flag: #todo. "the whole compilation machinery does not work with stdin directly, hence we load everything a buffer first"
	self evaluate: (String streamContents: [ :s|
		[ stdin atEnd ] whileFalse: [ 
			stdin next ifNotNilDo: [ :char|
			s nextPut: char ]]])! !

!CommandLineHandler class methodsFor: 'instance creation' stamp: 'CamilloBruni 5/2/2012 11:25'!
activateWith: aCommandLine 
	
	^ self new 
		commandLine: (self prepareSubcommand: aCommandLine);
		activate! !

!CommandLineHandler class methodsFor: 'accessing' stamp: 'CamilloBruni 4/29/2012 17:38'!
allHandlers
	^ self allSubclasses reject: [ :handler| handler isAbstract ]! !

!CommandLineHandler class methodsFor: 'instance creation' stamp: 'CamilloBruni 10/13/2012 13:18'!
commandLine: aCommandLine 
	
	^ self new 
		commandLine: aCommandLine;
		yourself! !

!CommandLineHandler class methodsFor: 'accessing' stamp: 'CamilloBruni 5/1/2012 13:56'!
commandName
	" Overwrite this method to provide a short name for this command handler.
	You may pass the commandName as first argument to the image to select this handler"
	^ self name! !

!CommandLineHandler class methodsFor: 'accessing' stamp: 'CamilloBruni 5/1/2012 13:53'!
description
	"This method should return a short one-line description of the command"
	^ nil! !

!CommandLineHandler class methodsFor: 'accessing' stamp: 'CamilloBruni 4/29/2012 17:39'!
isAbstract
	^ self = CommandLineHandler! !

!CommandLineHandler class methodsFor: 'handler selection' stamp: 'CamilloBruni 5/2/2012 14:06'!
isResponsibleFor: aCommandLineArguments
	^ aCommandLineArguments includesSubCommand: self commandName! !

!CommandLineHandler class methodsFor: 'private' stamp: 'CamilloBruni 5/2/2012 11:28'!
prepareSubcommand: commandLineArguments

	"strip the subcommand name from the arguments"
	commandLineArguments withFirstArgument: [ :arg|
		arg = self commandName 
			ifTrue: [ ^ commandLineArguments copySubcommand ]].
	
	"not a subcommand hence we keep the same args"
	^ commandLineArguments! !

!CommandLineHandler class methodsFor: 'handler selection' stamp: 'CamilloBruni 4/28/2012 01:51'!
priority
	^ 0! !

!CommandLineHandler class methodsFor: 'handler selection' stamp: 'CamilloBruni 5/2/2012 11:30'!
selectHandlersFor: aCommandLine

	^ self allHandlers select: [ :handlerClass|
		handlerClass isResponsibleFor: aCommandLine ]! !

!CommandLineHandler methodsFor: 'utility' stamp: 'CamilloBruni 4/29/2012 16:33'!
<< aString
	^ self stdout 
		nextPutAll: aString;
		yourself! !

!CommandLineHandler methodsFor: 'activation' stamp: 'CamilloBruni 4/28/2012 19:46'!
activate
	 self subclassResponsibility! !

!CommandLineHandler methodsFor: 'activation' stamp: 'CamilloBruni 10/13/2012 15:52'!
activateHelp
	"Default help implementation, running #help if the only argument is --help "
	((self hasOption: 'help') and: [ self arguments size = 1 ])
		ifTrue: [ ^ self help ].! !

!CommandLineHandler methodsFor: 'accessing' stamp: 'CamilloBruni 5/1/2012 13:43'!
allHandlers 
	^ CommandLineHandler allHandlers ! !

!CommandLineHandler methodsFor: 'accessing arguments' stamp: 'CamilloBruni 10/13/2012 13:13'!
argumentAt: anInteger
	^ self commandLine argumentAt: anInteger! !

!CommandLineHandler methodsFor: 'accessing arguments' stamp: 'CamilloBruni 10/13/2012 13:13'!
arguments
	^ self commandLine arguments! !

!CommandLineHandler methodsFor: 'accessing' stamp: 'CamilloBruni 4/28/2012 19:28'!
commandLine
	^ commandLine! !

!CommandLineHandler methodsFor: 'accessing' stamp: 'CamilloBruni 4/28/2012 19:28'!
commandLine: aCommandLine 
	commandLine := aCommandLine! !

!CommandLineHandler methodsFor: 'accessing' stamp: 'CamilloBruni 5/1/2012 13:42'!
commandName
	^ self class commandName! !

!CommandLineHandler methodsFor: 'accessing' stamp: 'CamilloBruni 5/1/2012 13:52'!
description
	^ self class description! !

!CommandLineHandler methodsFor: 'utility' stamp: 'CamilloBruni 10/13/2012 15:44'!
exitFailure 
	^ self exitFailure: 'Commandline failed'! !

!CommandLineHandler methodsFor: 'utility' stamp: 'CamilloBruni 10/13/2012 15:44'!
exitFailure: aMessage 
	Smalltalk isInteractive 
		ifTrue: [ Error signal: aMessage ]
		ifFalse: [ Smalltalk exitFailure ].! !

!CommandLineHandler methodsFor: 'utility' stamp: 'CamilloBruni 10/13/2012 16:06'!
exitSuccess
	Smalltalk isInteractive 
		ifTrue: [ self inform: self class name, ' finished' ]
		ifFalse: [ Smalltalk exitSuccess ]! !

!CommandLineHandler methodsFor: 'testing' stamp: 'CamilloBruni 10/13/2012 15:39'!
hasArguments
	^ self commandLine hasArguments! !

!CommandLineHandler methodsFor: 'accessing arguments' stamp: 'CamilloBruni 10/13/2012 13:13'!
hasOption: aString
	^ self commandLine hasOption: aString! !

!CommandLineHandler methodsFor: 'actions' stamp: 'CamilloBruni 10/13/2012 15:45'!
help
	"This is a crude default help implementation."
	self printHelp.
	Smalltalk isInteractive ifFalse: [ self exitSuccess ]! !

!CommandLineHandler methodsFor: 'initialize-release' stamp: 'CamilloBruni 10/13/2012 13:45'!
initialize
	super initialize.
	self 
		initializeStdout;
		initializeStderr.
	! !

!CommandLineHandler methodsFor: 'initialize-release' stamp: 'CamilloBruni 10/13/2012 13:32'!
initializeStderr
	"install the line end conversion and initialize the converter"
	FileStream stderr wantsLineEndConversion: true; converter.
	stderr := FileStream stderr .! !

!CommandLineHandler methodsFor: 'initialize-release' stamp: 'CamilloBruni 4/28/2012 19:47'!
initializeStdout
	"install the line end conversion and initialize the converter"
	FileStream stdout wantsLineEndConversion: true; converter.
	stdout := FileStream stdout.! !

!CommandLineHandler methodsFor: 'accessing arguments' stamp: 'CamilloBruni 10/13/2012 14:07'!
optionAt: aString
	^ self commandLine optionAt: aString! !

!CommandLineHandler methodsFor: 'printing' stamp: 'CamilloBruni 10/13/2012 15:42'!
printHelp
	self stderr
		nextPutAll: self class comment.! !

!CommandLineHandler methodsFor: 'utility' stamp: 'CamilloBruni 7/25/2012 10:53'!
quit
	Smalltalk exitSuccess.! !

!CommandLineHandler methodsFor: 'accessing' stamp: 'CamilloBruni 10/13/2012 13:31'!
stderr
	^ stderr! !

!CommandLineHandler methodsFor: 'accessing' stamp: 'CamilloBruni 4/28/2012 19:47'!
stdout
	^ stdout! !

CommandLineHandler subclass: #ConfigurationCommandLineHandler
	instanceVariableNames: 'version repositoryURL configurationName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-CommandLine'!

!ConfigurationCommandLineHandler commentStamp: '<historical>' prior: 0!
Usage: config [--help] <repository url> [<configuration>] [--install[=<version>]]
	--help              show this help message
	<repository url>    A Monticello repository name 
	<configuration>     A valid Metacello Configuration name
	<version>           A valid version for the given configuration
	
Examples:
	# display this help message
	$PharoVM My.image config
	
	# list all configurations of a repository
	$PharoVM My.image config $MC_REPOS_URL
	
	# list all the available versions of a confgurtation
	$PharoVM My.image config $MC_REPOS_URL ConfigurationOfFoo
	
	# install the stable version
	$PharoVM My.image config $MC_REPOS_URL ConfigurationOfFoo --install
	
	#install a specific version
	$PharoVM My.image config $MC_REPOS_URL ConfigurationOfFoo --install='1.5'!

!ConfigurationCommandLineHandler class methodsFor: 'accessing' stamp: 'CamilloBruni 10/13/2012 12:51'!
commandName
	^ 'config'! !

!ConfigurationCommandLineHandler class methodsFor: 'accessing' stamp: 'CamilloBruni 10/13/2012 12:51'!
description
	^ 'Install and inspect Metacello Configurations from the command line'! !

!ConfigurationCommandLineHandler methodsFor: 'activation' stamp: 'CamilloBruni 10/13/2012 15:50'!
activate
	self activateHelp.
	
	(self hasOption: 'install')
		ifFalse: [ ^ self list ].
		
	self installConfiguration.
		! !

!ConfigurationCommandLineHandler methodsFor: 'accessing' stamp: 'CamilloBruni 10/13/2012 13:55'!
configuration
	self gofer
		package: self configurationName;
		load.
	^ Smalltalk globals at: self configurationName asSymbol! !

!ConfigurationCommandLineHandler methodsFor: 'accessing' stamp: 'CamilloBruni 10/13/2012 13:54'!
configurationName
	^ configurationName ifNil: [ configurationName := self argumentAt: 2 ]! !

!ConfigurationCommandLineHandler methodsFor: 'actions' stamp: 'CamilloBruni 10/13/2012 15:46'!
errorNoConfigurationsFound

	self exitFailure: 'No Configurations found in ', repositoryURL asString! !

!ConfigurationCommandLineHandler methodsFor: 'accessing' stamp: 'CamilloBruni 10/13/2012 13:53'!
gofer
	^ Gofer new 
		url: self repositoryURL;
		yourself! !

!ConfigurationCommandLineHandler methodsFor: 'testing' stamp: 'CamilloBruni 10/13/2012 13:20'!
hasConfiguration
	self flag: 'TODO: should use proper cli parser Coral??'.
	^ (self arguments size > 1) and: [
		((self argumentAt: 2) beginsWith: '-') not ]! !

!ConfigurationCommandLineHandler methodsFor: 'actions' stamp: 'CamilloBruni 10/13/2012 14:12'!
installConfiguration
	self installVersion: ((self optionAt: 'install') ifNil: [ #stable ])! !

!ConfigurationCommandLineHandler methodsFor: 'actions' stamp: 'CamilloBruni 10/13/2012 15:36'!
installVersion: aVersionName

	self inform: 'Installing ', self configurationName, ' ', aVersionName.
	(self project version: aVersionName) load.
	
	Smalltalk snapshot: true andQuit: true.! !

!ConfigurationCommandLineHandler methodsFor: 'actions' stamp: 'CamilloBruni 10/13/2012 13:07'!
list
	self hasConfiguration 
		ifTrue: [ self listConfigurationDetails ]
		ifFalse: [ self listConfigurations ]! !

!ConfigurationCommandLineHandler methodsFor: 'actions' stamp: 'CamilloBruni 10/13/2012 14:02'!
listConfigurationDetails
	"List possible configurations of the given repository"
	
	self printMetacelloProjectDetails: self project.
	
	^ self exitSuccess! !

!ConfigurationCommandLineHandler methodsFor: 'actions' stamp: 'CamilloBruni 10/13/2012 13:42'!
listConfigurations
	"List possible configurations of the given repository"
	
	| configurations |
	
	configurations := self loadConfigurationNames.
	configurations ifEmpty: [ ^ self errorNoConfigurationsFound ].
	self printConfigurations: configurations.
	
	^ self exitSuccess! !

!ConfigurationCommandLineHandler methodsFor: 'actions' stamp: 'CamilloBruni 10/13/2012 13:53'!
loadConfigurationNames
	| references configurations |
	
	references := self gofer allResolved.
		
	references := references 
			collect: #packageName 
			thenSelect: [ :name| name beginsWith: 'ConfigurationOf' ].
	
	^ references asSet asSortedCollection ! !

!ConfigurationCommandLineHandler methodsFor: 'accessing' stamp: 'CamilloBruni 10/13/2012 13:25'!
loadRepositoryURL
	^ self argumentAt: 1.! !

!ConfigurationCommandLineHandler methodsFor: 'printing' stamp: 'CamilloBruni 10/13/2012 13:49'!
printConfigurations: configurations

	self inform: (String streamContents:  [ :s|
		s
			nextPutAll: 'Found ';
			print: configurations size;
			nextPutAll: ' Configuration';
			nextPutAll: (configurations size = 1 ifTrue: ':' ifFalse: 's:') ]).
	
	configurations do: [ :configurationName | 
			self << configurationName. 
			self stdout lf ].
	! !

!ConfigurationCommandLineHandler methodsFor: 'printing' stamp: 'CamilloBruni 10/13/2012 13:59'!
printMetacelloProjectDetails: metacelloProject
	self inform: 'Available versions for ', self configurationName.
	
	metacelloProject symbolicVersionMap keysDo: [ :key|
		self stdout print: key; lf ].
	
	metacelloProject map keysDo: [ :key|
		self stdout nextPutAll: key; lf ]! !

!ConfigurationCommandLineHandler methodsFor: 'accessing' stamp: 'CamilloBruni 10/13/2012 14:02'!
project
	^ self configuration project! !

!ConfigurationCommandLineHandler methodsFor: 'accessing' stamp: 'CamilloBruni 10/13/2012 13:24'!
repositoryURL
	^ repositoryURL ifNil: [ repositoryURL := self loadRepositoryURL ]! !

CommandLineHandler subclass: #DefaultCommandLineHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-CommandLine'!

!DefaultCommandLineHandler commentStamp: '<historical>' prior: 0!
Usage: [<subcommand>] [--help] [--copyright] [--version] [--list]
	--help       print this help message
	--copyright  print the copyrights
	--version    print the version for the image and the vm
	--list       list a description of all active command line handlers
	<subcommand> a valid subcommand in --list
	
Documentation:
A DefaultCommandLineHandler handles default command line arguments and options.
The DefaultCommandLineHandler is activated before all other handlers. 
It first checks if another handler is available. If so it will activate the found handler.!

!DefaultCommandLineHandler class methodsFor: 'handler selection' stamp: 'CamilloBruni 5/3/2012 14:13'!
description
	^ 'responsible for the default options and activating other commands'! !

!DefaultCommandLineHandler class methodsFor: 'class initialization' stamp: 'CamilloBruni 5/2/2012 11:34'!
initialize 
	"hook into the startup list"
	Smalltalk addToStartUpList: self.! !

!DefaultCommandLineHandler class methodsFor: 'handler selection' stamp: 'CamilloBruni 4/28/2012 01:27'!
isResponsibleFor: aCommandLine
	^ true! !

!DefaultCommandLineHandler class methodsFor: 'handler selection' stamp: 'CamilloBruni 5/2/2012 11:34'!
priority 
	"low priority"
	^ Float infinity negated! !

!DefaultCommandLineHandler class methodsFor: 'class initialization' stamp: 'CamilloBruni 5/1/2012 21:18'!
startUp: resuming

	"only handle when lauching a new image"
	resuming ifFalse: [ ^ self ].
	Smalltalk addDeferredStartupAction: [ self new activate ]! !

!DefaultCommandLineHandler methodsFor: 'activation' stamp: 'CamilloBruni 7/24/2012 16:38'!
activate
	self arguments ifEmpty: [ ^ self ].
	
	^ self handleArgument: self arguments first.! !

!DefaultCommandLineHandler methodsFor: 'commands' stamp: 'CamilloBruni 4/28/2012 21:54'!
copyright
	self stdout 
		nextPutAll: Smalltalk licenseString; cr.
	self quit.! !

!DefaultCommandLineHandler methodsFor: 'commands' stamp: 'CamilloBruni 10/13/2012 15:47'!
error
	self arguments size = 1
		ifTrue:  [
			(self arguments first beginsWith: '-')
				ifTrue:  [ self << 'Invalid option: ']
				ifFalse: [ self << 'Invalid sub command: ']]
		ifFalse: [ self << 'Invalid arguments: "'].
		
	(self << (' ' join: self arguments)) 
		nextPutAll: '"'; cr.
		
	self printHelp.
	
	self exitFailure.! !

!DefaultCommandLineHandler methodsFor: 'activation' stamp: 'CamilloBruni 5/2/2012 00:24'!
handleArgument: aString
	
	"give priority to subcommands"
	self handleSubcommand == self
		ifFalse: [ ^ self ].
		
	"check for default options"
	aString = '--version'
		ifTrue: [ ^ self version ].
		
	aString = '--help'
		ifTrue: [ ^ self help ].
		
	aString = '--list'
		ifTrue: [ ^ self list ].
		
	aString = '--copyright'
		ifTrue: [ ^ self copyright ].
	
	"none of the previous options matched hence we output an error message"
	self error.! !

!DefaultCommandLineHandler methodsFor: 'activation' stamp: 'CamilloBruni 5/2/2012 00:24'!
handleSubcommand
	"check if there is an active subcommand"
	
	| handlers |
	
	handlers := self selectHandlers.
	
	"ignore the default command line handler"
	handlers first = DefaultCommandLineHandler
		ifTrue: [ ^ self ].
		
	^ handlers first activateWith: commandLine.! !

!DefaultCommandLineHandler methodsFor: 'initialize-release' stamp: 'CamilloBruni 5/1/2012 21:24'!
initialize
	super initialize.
	commandLine := CommandLineArguments new! !

!DefaultCommandLineHandler methodsFor: 'activation' stamp: 'CamilloBruni 5/26/2012 13:09'!
list
	| handlers shortNames maxShortNameSize |
	self stdout 
		nextPutAll: 'Currently installed Command Line Handlers:'; cr.
		
	handlers := self allHandlers reject: [ :cls| cls = DefaultCommandLineHandler ].
	shortNames := handlers collect: #commandName.
	maxShortNameSize := (shortNames detectMax: [ :name|  name size ]) size.
	
	handlers do: [ :handler|
		self stdout 
			nextPutAll: '    ';
			nextPutAll: (handler commandName padRightTo: maxShortNameSize); 
			nextPutAll: '    ';
			nextPutAll: (handler description ifNil: [ '--']);cr].
	self quit.! !

!DefaultCommandLineHandler methodsFor: 'private' stamp: 'CamilloBruni 5/1/2012 21:22'!
selectHandlers
	| handlers |
	handlers := CommandLineHandler selectHandlersFor: commandLine.
	handlers := handlers sort: [ :a :b| a priority >= b priority ].
	^ handlers! !

!DefaultCommandLineHandler methodsFor: 'commands' stamp: 'CamilloBruni 4/28/2012 22:00'!
version
	self stdout 
		nextPutAll: 'Image: '; print: SystemVersion current; cr;
		nextPutAll: 'VM:    '; nextPutAll: Smalltalk vm version; cr.
	self quit.! !

CommandLineHandler subclass: #SaveCommandLineHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-CommandLine'!

!SaveCommandLineHandler commentStamp: '<historical>' prior: 0!
Usage: save <imageBaseName> [--delete-old]
 	<imageName>     a base name for the image
	--delete-old    remove the old image and changes file
	
Documentation:
Saves the image and changes file under a new name.

Examples:
	# create a 'bar.image' and 'foo.changes'
	$PHAROVM Foo.image saveAs bar
	# create the same file as in the previous example but delete Foo.image and Foo.changes
	$PHAROVM Foo.image saveAs bar --delete-old!

!SaveCommandLineHandler class methodsFor: 'accessing' stamp: 'CamilloBruni 10/18/2012 11:03'!
commandName
	^ 'save'! !

!SaveCommandLineHandler class methodsFor: 'accessing' stamp: 'CamilloBruni 10/18/2012 11:05'!
description
	^ 'Rename the image and changes file'! !

!SaveCommandLineHandler methodsFor: 'activation' stamp: 'CamilloBruni 10/18/2012 11:36'!
activate
	self activateHelp.
	
	((self hasOption: 'delete-old') 
		ifTrue: [ self arguments size = 2 ]
		ifFalse: [ self arguments size = 1])
			ifFalse: [ ^ self exitFailure: 'Invalid Arguments' ].
		
	self saveImage.
		! !

!SaveCommandLineHandler methodsFor: 'actions' stamp: 'CamilloBruni 10/18/2012 11:33'!
saveImage
	| imageName oldImageFile oldChangesFile res |
	
	imageName := self arguments detect: [ :arg| arg ~= '--delete-old' ].
	oldImageFile := Smalltalk imageFile resolve.
	oldChangesFile := Smalltalk changesFile resolve.
	
	(Smalltalk saveAs: imageName)
		ifTrue: [ 
			"rebooting, when executing this, we are already in the renamed image" 
			^ self ].
	
	"...otherwise continuing"
	(self hasOption: 'delete-old')
		ifFalse: [ ^ self exitSuccess ].
	
	oldImageFile delete.
	oldChangesFile delete.
	
	self exitSuccess.! !

CommandLineHandler subclass: #TestRunnerCommandLineHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-CommandLine'!

!TestRunnerCommandLineHandler commentStamp: '<historical>' prior: 0!
Usage: test [<package> ...]
	<package> a String matching a package name
	
Examples:
	#Run all the tests in the Tests-Exceptions package
	$PharoVM Foo.image test Tests-Exceptions
	#Run all the tests in packages matching Test-.*
	$PharoVM Foo.image test "Tests-.*"
	!

!TestRunnerCommandLineHandler class methodsFor: 'accessing' stamp: 'CamilloBruni 10/18/2012 12:52'!
commandName
	^ 'test'! !

!TestRunnerCommandLineHandler class methodsFor: 'accessing' stamp: 'CamilloBruni 10/18/2012 12:52'!
description
	^ 'A command line test runner'! !

!TestRunnerCommandLineHandler methodsFor: 'activation' stamp: 'CamilloBruni 10/18/2012 13:01'!
activate
	self activateHelp.
	self runPackages.! !

!TestRunnerCommandLineHandler methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 10/18/2012 13:21'!
addPackagesMatching: aString to: aSet
	| regex |
	
	[ regex := aString asRegex ] 
		on: Error do: [ ].
	regex ifNotNil: [ ^ self addPackagesMatchingRegex: regex to: aSet ].
	
	(aString includes: $*)
		ifTrue: [ ^ self addPackagesMatchingGlob: aString to: aSet ].
	
	"exact match"
	(RPackage organizer includesPackageNamed: aString)
		ifTrue: [ aSet add: aString ]! !

!TestRunnerCommandLineHandler methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 10/18/2012 13:18'!
addPackagesMatchingGlob: aGlobString to: aSet
	RPackage organizer packageNames do: [ :packageName|
		(aGlobString match: packageName)
			ifTrue: [ aSet add: packageName ]]! !

!TestRunnerCommandLineHandler methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 10/18/2012 13:16'!
addPackagesMatchingRegex: aRegex to: aSet
	RPackage organizer packageNames do: [ :packageName|
		(aRegex matches: packageName)
			ifTrue: [ aSet add: packageName ]]! !

!TestRunnerCommandLineHandler methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 10/18/2012 13:10'!
packages
	| packages |
	packages := Set new.
	self arguments 
		reject: [ :arg| arg beginsWith: '-']
		thenDo: [ :arg| self addPackagesMatching: arg to: packages ].
	^ packages! !

!TestRunnerCommandLineHandler methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 10/18/2012 18:40'!
runPackages
	| packages |
	packages := self packages.
	self inform: 'Running tests in ', packages size asString, ' Packages'.
	self testRunner runPackages: self packages.
	self exitSuccess.! !

!TestRunnerCommandLineHandler methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 10/18/2012 18:04'!
testRunner
	^ (self hasOption: '--junit-xml-output')
		ifTrue: [ HDTestReport ]
		ifFalse: [ CommandLineTestRunner ]! !

Object subclass: #Termcap
	instanceVariableNames: 'cache'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-CommandLine'!

!Termcap commentStamp: '<historical>' prior: 0!
I provide a simple mapping from high-level commands to low-level terminal character sequences.
!

!Termcap methodsFor: 'initialize-release' stamp: 'dfgs 7/8/2012 14:01'!
flush
	cache removeAll! !

!Termcap methodsFor: 'accessing' stamp: 'CamilloBruni 10/19/2012 12:44'!
getent: aString
	"The tgetent routine loads the entry for name. 
	It returns 1 on success, 
	0 if there is no such entry,
	 and -1 if the terminfo database could not be found."
	| result |
	
	"result := (PipeableOSProcess command: 'tput ', aString) output asInteger.
	result = 1 ifTrue: [ ^ self ].
	result = 0 ifTrue: [ Error signal: 'No such tget entry: ', aString aString ].
	result = -1 ifTrue: [ Error signal: 'Could not find terminfo database' ]."
	
	Error signal: 'Cannot interact with tput yet'.
	! !

!Termcap methodsFor: 'accessing' stamp: 'CamilloBruni 10/19/2012 12:43'!
getflag: aString
	"^ (PipeableOSProcess command: 'tput ', aString) output"
	Error signal: 'Cannot interact with tput yet'! !

!Termcap methodsFor: 'accessing' stamp: 'CamilloBruni 10/19/2012 12:42'!
getnum: aString
	"^ (PipeableOSProcess command: 'tput ', aString) output asInteger"
	^ Error signal: 'Cannot interact with the terminal yet'.! !

!Termcap methodsFor: 'accessing' stamp: 'CamilloBruni 10/19/2012 12:42'!
getstr: aString
	^ cache
		at: aString 
		ifAbsentPut: [ 
			"(PipeableOSProcess command: 'tput ', aString) output"
			^ Error signal: 'Terminal capability ', aString, ' not found' ]! !

!Termcap methodsFor: 'initialize-release' stamp: 'CamilloBruni 10/19/2012 14:45'!
initialize
	cache := Dictionary new.
	self initializeDefaultSequences.! !

!Termcap methodsFor: 'initialize-release' stamp: 'CamilloBruni 10/19/2012 14:50'!
initializeDefaultSequences
	"Until we have decent OSProcess support we precache the most common sequences"
	| esc |
	esc := (Character escape asString), '['.
	cache at: #up put: (esc, 'A').
	cache at: #le put: (esc, 'D').
	cache at: #nd put: (esc, 'C').
	cache at: #im put: ''.
	cache at: #ei put: ''.
	cache at: #mr put: (esc, '7m').
	cache at: #me put: (esc, '0m').
	cache at: #ce put: (esc, 'K').
	cache at: #al put: (esc, 'L').
	cache at: #dl put: (esc, 'M').
	cache at: #cr put: (esc, String cr).! !

!Termcap methodsFor: 'initialize-release' stamp: 'CamilloBruni 10/19/2012 12:45'!
reset
	self flag: 'TODO implement once we have OSProcess'.
	"self getent: (OSProcess thisOSProcess environmentAt: #TERM)"! !

Object subclass: #VTermInputDriver
	instanceVariableNames: 'inStream handler normal csi ss3'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-CommandLine'!

!VTermInputDriver commentStamp: '<historical>' prior: 0!
I convert low-level terminal inputs, like mouse report or a key command.!

!VTermInputDriver class methodsFor: 'instance creation' stamp: 'CamilloBruni 9/29/2011 18:32'!
on: inputStream
	^ self new	
		inStream: inputStream;
		yourself! !

!VTermInputDriver class methodsFor: 'instance creation' stamp: 'dfgs 7/6/2012 16:56'!
stdIn
	^ self on: FileStream stdin! !

!VTermInputDriver methodsFor: 'reading' stamp: 'CamilloBruni 10/4/2011 22:54'!
continueReadCSINumber: v
	| value character |
	value := v.
	[ $0 <= (character := inStream next) and: [ character <= $9 ]]
		whileTrue: [
			value := value * 10 + (character asInteger - $0 asInteger ) ].
	character = $~ ifFalse: [ self fail: character in: 'Control Sequence Introducer (number end)' ].
	^ csi at: value ifAbsent: [ self fail: character in: 'Control Sequence Introducer (number end)' ].! !

!VTermInputDriver methodsFor: 'private' stamp: 'CamilloBruni 3/28/2012 19:56'!
fail: value in: mode
	self error: mode asString, ' does not support ', value asString! !

!VTermInputDriver methodsFor: 'accessing' stamp: 'ToonVerwaest 8/20/2010 14:15'!
handler: anObject
	handler := anObject! !

!VTermInputDriver methodsFor: 'accessing' stamp: 'CamilloBruni 8/11/2011 19:38'!
inStream: aStream
	inStream := aStream! !

!VTermInputDriver methodsFor: 'initialize-release' stamp: 'CamilloBruni 9/29/2011 18:32'!
initialize

	normal := Dictionary new.
	self initializeNormal: normal.
	
	ss3 := Dictionary new.
	self initializeSS3: ss3.
	
	csi := Dictionary new.
	self initializeCSI: csi! !

!VTermInputDriver methodsFor: 'initialize-release' stamp: 'dfgs 7/8/2012 13:08'!
initializeCSI: actions
	actions at: $A put: [ handler up ].
	actions at: $B put: [ handler down ].
	actions at: $C put: [ handler right ].
	actions at: $D put: [ handler left ].
	actions at: $E put: [ handler here ].
	actions at: $F put: [ handler pageUp ].
	actions at: $H put: [ handler pageDown ].
	
	actions at: 1 put: [ handler find ].
	actions at: 2 put: [ handler insert ].
	actions at: 3 put: [ handler remove ].
	actions at: 4 put: [ handler select ].
	actions at: 5 put: [ handler previousScreen ].
	actions at: 6 put: [ handler nextScreen ].
	
	actions at: 15 put: [ handler functionKey: 5 ].
	actions at: 17 put: [ handler functionKey: 6 ].
	actions at: 18 put: [ handler functionKey: 7 ].
	actions at: 19 put: [ handler functionKey: 8 ].
	actions at: 20 put: [ handler functionKey: 9 ].
	actions at: 21 put: [ handler functionKey: 10 ].
	actions at: 23 put: [ handler functionKey: 11 ].
	actions at: 24 put: [ handler functionKey: 12 ].
	actions at: 25 put: [ handler functionKey: 13 ].
	actions at: 26 put: [ handler functionKey: 14 ].
	actions at: 28 put: [ handler functionKey: 15 ].
	actions at: 29 put: [ handler functionKey: 16 ].
	actions at: 31 put: [ handler functionKey: 17 ].
	actions at: 32 put: [ handler functionKey: 18 ].
	actions at: 33 put: [ handler functionKey: 19 ].
	actions at: 34 put: [ handler functionKey: 20 ].
	
	actions at: $[ put: [ self readCSI ].! !

!VTermInputDriver methodsFor: 'initialize-release' stamp: 'dfgs 7/8/2012 13:08'!
initializeNormal: actions
	actions at: 1 put: [ handler startOfHeading ].
	actions at: 2 put: [ handler startOfText ].
	actions at: 3 put: [ handler endOfText ].
	actions at: 4 put: [ handler endOfTransmission ].
	actions at: 5 put: [ handler enquiry ].
	actions at: 6 put: [ handler acknowledge ].
	actions at: 7 put: [ handler bell ].
	actions at: 8 put: [ handler backspace ].
	actions at: 9 put: [ handler tab ].
	actions at: 10 put: [ handler lf ].
	actions at: 11 put: [ handler verticalTab ].
	actions at: 12 put: [ handler formfeed ].
	actions at: 13 put: [ handler carriageReturn ].
	actions at: 14 put: [ handler shiftOut ].
	actions at: 15 put: [ handler shiftIn ].
	actions at: 27 put: [ self readC1 ].
	actions at: 127 put: [ handler backspace ].! !

!VTermInputDriver methodsFor: 'initialize-release' stamp: 'dfgs 7/8/2012 13:07'!
initializeSS3: actions
	actions at: $p put: [ handler keypad: 0 ].
	actions at: $q put: [ handler keypad: 1 ].
	actions at: $r put: [ handler keypad: 2 ].
	actions at: $s put: [ handler keypad: 3 ].
	actions at: $t put: [ handler keypad: 4 ].
	actions at: $u put: [ handler keypad: 5 ].
	actions at: $v put: [ handler keypad: 6 ].
	actions at: $w put: [ handler keypad: 7 ].
	actions at: $x put: [ handler keypad: 8 ].
	actions at: $y put: [ handler keypad: 9 ].
	
	actions at: $j put: [ handler keypadTimes ].
	actions at: $k put: [ handler keypadPlus ].
	actions at: $l put: [ handler keypadBacktick ].
	actions at: $n put: [ handler keypadDot ].
	actions at: $M put: [ handler keypadEnter ].
	actions at: $m put: [ handler keypadMinus ].
	actions at: $o put: [ handler keypadDivide ].
	actions at: $X put: [ handler keypadEqual ].
	
	actions at: $P put: [ handler functionKey: 1 ].
	actions at: $Q put: [ handler functionKey: 2 ].
	actions at: $R put: [ handler functionKey: 3 ].
	actions at: $S put: [ handler functionKey: 4 ].
	actions at: $H put: [ handler home ].
	actions at: $F put: [ handler end ].! !

!VTermInputDriver methodsFor: 'reading' stamp: 'DamienPollet 10/6/2011 16:40'!
read
	" C0 Control (Escape) mode "
	| character action |
	character := inStream next.
	character ifNil: [ Error signal: 'could not read character from input' ].
	action := normal
				at: character asInteger
				ifAbsent: [ ^ handler write: character ].
	action value! !

!VTermInputDriver methodsFor: 'reading' stamp: 'CamilloBruni 10/3/2011 16:09'!
readC1
	" C1 Control Escape mode "
	| character |
	character := inStream next.
	character = $[ ifTrue: [ ^ self readCSI ].
	character = $O ifTrue: [ ^ self readSS3 ].
	handler alt: character! !

!VTermInputDriver methodsFor: 'reading' stamp: 'CamilloBruni 10/3/2011 16:10'!
readCSI
	" Control Sequence Introducer mode "
	| code character action |
	character := inStream next.
	action := ($0 <= character and: [ character <= $9 ])
		ifTrue: [ self continueReadCSINumber: character asInteger - $0 asInteger ]
		ifFalse: [
			csi at: character ifAbsent: [
				self fail: character in: 'Control Sequence Introducer' ]].
	action value! !

!VTermInputDriver methodsFor: 'reading' stamp: 'CamilloBruni 6/29/2012 10:06'!
readSS3
	" Single-Shift 3 mode "
	| character action |
	character := inStream next.
	action := ss3 at: character ifAbsent: [ 
		self fail: character in: 'Single-Shift 3' ].
	action value! !

Object subclass: #VTermOutputDriver
	instanceVariableNames: 'termcap outStream currentBackground currentColor light blink shouldEcho'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-CommandLine'!

VTermOutputDriver class
	instanceVariableNames: 'stdoutTerminalInstance stderrTerminalInstance'!

!VTermOutputDriver commentStamp: '<historical>' prior: 0!
I convert high-level terminal commands like moving the cursor or setting the color to characters sequences understood by the terminal.

Furthermore I do some bookkeeping to reduce the number of characters sent to the terminal.!

VTermOutputDriver class
	instanceVariableNames: 'stdoutTerminalInstance stderrTerminalInstance'!

!VTermOutputDriver class methodsFor: 'instance-creation' stamp: 'CamilloBruni 9/29/2011 18:30'!
on: anOutputStream
	^ self new
		outStream: anOutputStream;
		yourself! !

!VTermOutputDriver class methodsFor: 'examples' stamp: 'CamilloBruni 7/1/2012 15:35'!
printText: aText
	" Example code shows how to output formatted text"
	| out |

	out := self stdOut.

	aText doWithIndex: [ :char :index| 
		aText attributesAt: index do: [ :attr| 
			(attr isKindOf: TextEmphasis )	ifTrue: [ out bold ].
			(attr isKindOf: TextColor )	ifTrue: [ out color256: attr color ]].
		(char = Character cr)
			ifTrue: [ out lf ]
			ifFalse: [ out << char ].
		out clear ].

	out lf.! !

!VTermOutputDriver class methodsFor: 'instance-creation' stamp: 'CamilloBruni 10/19/2012 12:46'!
stderr
	^ self on: FileStream stderr! !

!VTermOutputDriver class methodsFor: 'instance-creation' stamp: 'CamilloBruni 10/19/2012 12:46'!
stdout
	^ self on: FileStream stdout! !

!VTermOutputDriver methodsFor: 'writing' stamp: 'dfgs 7/8/2012 13:24'!
<< aStringOrCharacter
	aStringOrCharacter putOn: outStream.! !

!VTermOutputDriver methodsFor: 'coloring' stamp: 'CamilloBruni 7/1/2012 14:19'!
background256: aColor
	"output an xterm 256 color extension sequence"
	self color256: aColor background: true! !

!VTermOutputDriver methodsFor: 'removing' stamp: 'dfgs 7/8/2012 13:24'!
backspace
	outStream nextPut: Character backspace.
	self deleteCharacter ! !

!VTermOutputDriver methodsFor: 'coloring' stamp: 'CamilloBruni 5/31/2010 22:52'!
black
	self color: 30.! !

!VTermOutputDriver methodsFor: 'coloring' stamp: 'CamilloBruni 5/31/2010 23:18'!
black: aString
	self black; << aString; flush; clear.! !

!VTermOutputDriver methodsFor: 'highlighting' stamp: 'dfgs 7/8/2012 13:26'!
blinking
	self termcap: 'mb'! !

!VTermOutputDriver methodsFor: 'coloring' stamp: 'CamilloBruni 5/31/2010 22:46'!
blue
	self color: 34.! !

!VTermOutputDriver methodsFor: 'coloring' stamp: 'CamilloBruni 5/31/2010 23:18'!
blue: aString
	self blue; << aString; flush; clear.! !

!VTermOutputDriver methodsFor: 'highlighting' stamp: 'dfgs 7/8/2012 13:25'!
bold
	" double bright mode "
	self termcap: 'md'! !

!VTermOutputDriver methodsFor: 'accessing' stamp: 'CamilloBruni 6/1/2010 00:06'!
clear
	self reset.
	self colorEscape.! !

!VTermOutputDriver methodsFor: 'removing' stamp: 'dfgs 7/8/2012 13:26'!
clearFromBeginning
	self termcap: 'cb'! !

!VTermOutputDriver methodsFor: 'removing' stamp: 'dfgs 7/8/2012 13:26'!
clearScreen
	self termcap: 'cl'! !

!VTermOutputDriver methodsFor: 'removing' stamp: 'dfgs 7/8/2012 13:26'!
clearToEnd
	self termcap: 'ce'! !

!VTermOutputDriver methodsFor: 'coloring' stamp: 'CamilloBruni 10/19/2012 12:49'!
color256: aColor
	"output an xterm 256 color extension sequence."
	self color256: aColor background: false! !

!VTermOutputDriver methodsFor: 'coloring' stamp: 'CamilloBruni 10/19/2012 12:49'!
color256: aColor background: isBackground
	"use the resources for colors 0-15 - usually more-or-less a reproduction of the standard ANSI colors, but possibly more pleasing shades

	colors 16-231 are a 6x6x6 color cube

	colors 232-255 are a grayscale ramp, intentionally leaving out black and white"
	
	| colorIndex |
	
	self csiEscape.

	outStream nextPutAll: (isBackground 
		ifFalse: [ '38;5;' ]
		ifTrue:  [ '48;5;' ]).
		
	outStream 
		print: aColor closestXTermPixelValue;
		nextPut: $m.! !

!VTermOutputDriver methodsFor: 'coloring' stamp: 'CamilloBruni 5/31/2010 23:57'!
color: aColor
	currentColor := aColor.
	self colorEscape.! !

!VTermOutputDriver methodsFor: 'escaping' stamp: 'CamilloBruni 10/19/2012 12:47'!
colorEscape
	self csiEscape.
	blink ifTrue: [ outStream nextPutAll: '5;' ].
	outStream print: currentColor.
	(currentBackground == 0) ifFalse: [
		outStream nextPut: $;; print: currentBackground ].
	outStream nextPut: $m.! !

!VTermOutputDriver methodsFor: 'accessing' stamp: 'ToonVerwaest 8/16/2010 16:09'!
columns
	^ termcap getnum: 'co'! !

!VTermOutputDriver methodsFor: 'navigating' stamp: 'dfgs 7/8/2012 13:27'!
cr
	self termcap: 'cr'! !

!VTermOutputDriver methodsFor: 'escaping' stamp: 'dfgs 7/8/2012 13:17'!
csiEscape
	"Control Sequence Introducer escape"
	outStream nextPut: Character escape; nextPut: $[! !

!VTermOutputDriver methodsFor: 'cursor' stamp: 'dfgs 7/8/2012 13:27'!
cursorEnhanced
	self termcap: 'vs'! !

!VTermOutputDriver methodsFor: 'cursor' stamp: 'dfgs 7/8/2012 13:27'!
cursorInvisible
	self termcap: 'vi'! !

!VTermOutputDriver methodsFor: 'cursor' stamp: 'dfgs 7/8/2012 13:27'!
cursorNormal
	self termcap: 've'! !

!VTermOutputDriver methodsFor: 'coloring' stamp: 'CamilloBruni 5/31/2010 22:47'!
cyan
	self color: 36.! !

!VTermOutputDriver methodsFor: 'coloring' stamp: 'CamilloBruni 5/31/2010 23:19'!
cyan: aString
	self cyan; << aString; flush; clear.! !

!VTermOutputDriver methodsFor: 'removing' stamp: 'dfgs 7/8/2012 13:27'!
deleteCharacter
	self termcap: 'dc'! !

!VTermOutputDriver methodsFor: 'removing' stamp: 'dfgs 7/8/2012 13:27'!
deleteLine
	self termcap: 'dl'! !

!VTermOutputDriver methodsFor: 'navigating' stamp: 'dfgs 7/8/2012 13:29'!
down
	self termcap: 'do'! !

!VTermOutputDriver methodsFor: 'navigating' stamp: 'CamilloBruni 10/5/2011 01:04'!
down: distance
	distance < 0
		ifTrue: [ 0 - distance timesRepeat: [ self up ]]
		ifFalse: [ distance timesRepeat: [ self down ]].! !

!VTermOutputDriver methodsFor: 'writing' stamp: 'CamilloBruni 6/14/2010 18:15'!
echo
	self echo: true.! !

!VTermOutputDriver methodsFor: 'writing' stamp: 'CamilloBruni 6/14/2010 18:15'!
echo: aBool
	shouldEcho = aBool ifTrue: [ ^ self ].
	shouldEcho := aBool.
	self enable: #echo.! !

!VTermOutputDriver methodsFor: 'mode' stamp: 'dfgs 7/8/2012 13:29'!
endInsertMode
	self termcap: 'ei'! !

!VTermOutputDriver methodsFor: 'removing' stamp: 'CamilloBruni 10/19/2012 12:52'!
erase
	"outStream nextPutAll:
		(termcap parm: '1' in:
			(termcap getstr: 'ec'))"
	self shouldBeImplemented.! !

!VTermOutputDriver methodsFor: 'coloring' stamp: 'ToonVerwaest 8/17/2010 14:01'!
errorColor 
	self red.
	self reverse! !

!VTermOutputDriver methodsFor: 'escaping' stamp: 'dfgs 7/8/2012 13:18'!
escape
	outStream nextPut: Character escape! !

!VTermOutputDriver methodsFor: 'writing' stamp: 'ToonVerwaest 8/6/2010 16:51'!
flush
	outStream flush.! !

!VTermOutputDriver methodsFor: 'coloring' stamp: 'CamilloBruni 5/31/2010 22:47'!
green
	self color: 32.! !

!VTermOutputDriver methodsFor: 'coloring' stamp: 'CamilloBruni 5/31/2010 23:19'!
green: aString
	self green; << aString; flush; clear.! !

!VTermOutputDriver methodsFor: 'navigating' stamp: 'dfgs 7/8/2012 13:30'!
here
	self flag: #todo. "use termcap here"
	self csiEscape.
	outStream nextPut: $E! !

!VTermOutputDriver methodsFor: 'navigating' stamp: 'dfgs 7/8/2012 13:30'!
home
	self termcap: 'ho'! !

!VTermOutputDriver methodsFor: 'initialization' stamp: 'CamilloBruni 10/19/2012 11:26'!
initialize 
	super initialize.
	termcap := Termcap new.
	self reset.! !

!VTermOutputDriver methodsFor: 'inserting' stamp: 'dfgs 7/8/2012 13:30'!
insertCharacter
	self termcap:  'ic'! !

!VTermOutputDriver methodsFor: 'inserting' stamp: 'dfgs 7/8/2012 13:30'!
insertLine
	self termcap: 'al'.
	self lf! !

!VTermOutputDriver methodsFor: 'mode' stamp: 'dfgs 7/8/2012 13:30'!
insertMode
	self termcap: 'im'! !

!VTermOutputDriver methodsFor: 'highlighting' stamp: 'dfgs 7/8/2012 13:31'!
invisible
	self termcap: 'mk'! !

!VTermOutputDriver methodsFor: 'navigating' stamp: 'dfgs 7/8/2012 13:31'!
left
	self termcap:  'le'! !

!VTermOutputDriver methodsFor: 'navigating' stamp: 'CamilloBruni 10/4/2011 19:15'!
left: distance
	distance < 0
		ifTrue: [ 0 - distance timesRepeat: [ self right ]]
		ifFalse: [ distance timesRepeat: [ self left ]].! !

!VTermOutputDriver methodsFor: 'writing' stamp: 'ToonVerwaest 8/6/2010 16:51'!
lf
	outStream lf! !

!VTermOutputDriver methodsFor: 'coloring' stamp: 'CamilloBruni 5/31/2010 23:36'!
light
	light := true.! !

!VTermOutputDriver methodsFor: 'accessing' stamp: 'ToonVerwaest 8/16/2010 16:09'!
lines
	^ termcap getnum: 'li'! !

!VTermOutputDriver methodsFor: 'printing' stamp: 'abc 10/19/2012 15:50'!
nextPut: aCharacter 
	outStream nextPut: aCharacter.! !

!VTermOutputDriver methodsFor: 'printing' stamp: 'abc 10/19/2012 15:24'!
nextPutAll: aString
	outStream nextPutAll: aString! !

!VTermOutputDriver methodsFor: 'writing' stamp: 'CamilloBruni 6/14/2010 18:15'!
noEcho
	self echo: false.! !

!VTermOutputDriver methodsFor: 'highlighting' stamp: 'dfgs 7/8/2012 13:31'!
normal
	self termcap: 'me'! !

!VTermOutputDriver methodsFor: 'accessing' stamp: 'CamilloBruni 5/31/2010 17:40'!
outStream
	^ outStream! !

!VTermOutputDriver methodsFor: 'accessing' stamp: 'CamilloBruni 5/31/2010 17:40'!
outStream: stream
	outStream := stream.! !

!VTermOutputDriver methodsFor: 'writing' stamp: 'dfgs 7/8/2012 13:23'!
overwrite: char
	self flag: #TODO. "properly implement this"
	outStream nextPut: char! !

!VTermOutputDriver methodsFor: 'coloring' stamp: 'CamilloBruni 5/31/2010 22:48'!
pink
	self color: 35.! !

!VTermOutputDriver methodsFor: 'coloring' stamp: 'dfgs 7/8/2012 13:18'!
pink: aString
	self pink; nextPutAll: aString; flush; clear.! !

!VTermOutputDriver methodsFor: 'printing' stamp: 'abc 10/19/2012 15:23'!
print: anObject 
	anObject printOn: self! !

!VTermOutputDriver methodsFor: 'coloring' stamp: 'CamilloBruni 5/31/2010 22:49'!
red
	self color: 31.! !

!VTermOutputDriver methodsFor: 'coloring' stamp: 'dfgs 7/8/2012 13:22'!
red: aString
	self red; << aString; flush; clear.! !

!VTermOutputDriver methodsFor: 'coloring' stamp: 'ToonVerwaest 8/16/2010 16:23'!
reset
	currentColor := 0.
	currentBackground := 0.
	light := false.
	blink := false.! !

!VTermOutputDriver methodsFor: 'navigating' stamp: 'dfgs 7/8/2012 13:31'!
restoreCursor
	self termcap: 'rc'! !

!VTermOutputDriver methodsFor: 'highlighting' stamp: 'dfgs 7/8/2012 13:31'!
reverse
	self termcap: 'mr'! !

!VTermOutputDriver methodsFor: 'navigating' stamp: 'dfgs 7/8/2012 13:31'!
right
	self termcap: 'nd'! !

!VTermOutputDriver methodsFor: 'navigating' stamp: 'CamilloBruni 10/4/2011 19:16'!
right: distance
	distance < 0
		ifTrue: [ 0 - distance timesRepeat: [ self left ]]
		ifFalse: [ distance timesRepeat: [ self right ]].! !

!VTermOutputDriver methodsFor: 'navigating' stamp: 'dfgs 7/8/2012 13:19'!
saveCursor
	outStream nextPutAll: (termcap getstr: 'sc')! !

!VTermOutputDriver methodsFor: 'scrolling' stamp: 'dfgs 7/8/2012 13:32'!
scrollBackward
	self termcap: 'sr'! !

!VTermOutputDriver methodsFor: 'scrolling' stamp: 'dfgs 7/8/2012 13:33'!
scrollForward
	self termcap: 'sf'! !

!VTermOutputDriver methodsFor: 'writing' stamp: 'ToonVerwaest 8/6/2010 16:59'!
space
	outStream space! !

!VTermOutputDriver methodsFor: 'escaping' stamp: 'dfgs 7/8/2012 13:19'!
ss3Escape
	"Control Sequence Introducer escape"
	outStream nextPut: Character escape; nextPut: $O! !

!VTermOutputDriver methodsFor: 'navigating' stamp: 'dfgs 7/8/2012 14:22'!
startOfLine
	^ self cr! !

!VTermOutputDriver methodsFor: 'writing' stamp: 'ToonVerwaest 8/6/2010 16:59'!
tab
	outStream tab! !

!VTermOutputDriver methodsFor: 'accessing' stamp: 'dfgs 7/8/2012 14:08'!
termcap
	^ termcap! !

!VTermOutputDriver methodsFor: 'private' stamp: 'dfgs 7/8/2012 14:26'!
termcap: termcapCapabilityIdentifier
	"Output the termcap command sequence for the identifier.
	Use #basicNextPutAll: to avoid possible conversions applied by the stream"
	outStream basicNextPutAll: (termcap getstr: termcapCapabilityIdentifier)! !

!VTermOutputDriver methodsFor: 'navigating' stamp: 'dfgs 7/8/2012 13:33'!
topLeft
	self termcap: 'ho'! !

!VTermOutputDriver methodsFor: 'highlighting' stamp: 'dfgs 7/8/2012 13:33'!
underline
	self termcap: 'us'! !

!VTermOutputDriver methodsFor: 'highlighting' stamp: 'dfgs 7/8/2012 13:33'!
underlineOff
	self termcap: 'ue'! !

!VTermOutputDriver methodsFor: 'navigating' stamp: 'dfgs 7/8/2012 13:33'!
up
	self termcap: 'up'! !

!VTermOutputDriver methodsFor: 'navigating' stamp: 'CamilloBruni 10/4/2011 19:16'!
up: distance
	distance < 0
		ifTrue: [ 0 - distance timesRepeat: [ self down ]]
		ifFalse: [ distance timesRepeat: [ self up ]].! !

!VTermOutputDriver methodsFor: 'coloring' stamp: 'CamilloBruni 5/31/2010 22:50'!
white
	self color: 37.! !

!VTermOutputDriver methodsFor: 'coloring' stamp: 'CamilloBruni 5/31/2010 23:19'!
white: aString
	self white; << aString; flush; clear.! !

!VTermOutputDriver methodsFor: 'accessing' stamp: 'dfgs 7/8/2012 14:04'!
width
	^ self columns! !

!VTermOutputDriver methodsFor: 'coloring' stamp: 'CamilloBruni 5/31/2010 22:50'!
yellow
	self color: 33.! !

!VTermOutputDriver methodsFor: 'coloring' stamp: 'CamilloBruni 5/31/2010 23:19'!
yellow: aString
	self yellow; << aString; flush; clear.! !
DefaultCommandLineHandler initialize!
